/* 
 * Web Switch (UR7HFO)
 *
 * "ENC28J60 Module" Arduino Uno. 
 * VCC -   3.3V
 * GND -    GND
 * SCK - Pin 13
 * SO  - Pin 12
 * SI  - Pin 11
 * CS  - Pin 10 
 * 
 * Relay pins - 2,3,4,5,6,7,8,9,A5
 * 
 * A4 - temperature sensor. 
 * A3 - SWR_1 forward 
 * A2 - SWR_1 reflection
 * A1 - SWR_2 forward  
 * A0 - SWR_2 reflection
*/

#include <avr/pgmspace.h>
#include <EtherCard.h>
#include <avr/wdt.h>

// Uncomment line to enable $T in emit_P for float emitting (from file: EtherCard.cpp) 
#define FLOATEMIT 

// Uncomment for inverted relay module 
// #define INVERTED_RELAY

#ifdef INVERTED_RELAY
  #define ON    false
  #define OFF   true
#else
  #define ON    true
  #define OFF   false
#endif
  
#define R1  2
#define R2  3
#define R3  4
#define R4  5
#define R5  6
#define R6  7
#define R7  8
#define R8  9
#define R9  A5

// MAC Address.
static byte mymac[] = { 0x5A,0x5A,0x5A,0x5A,0x5A,0x5A };

byte Ethernet::buffer[1000];
BufferFiller bfill;

boolean PinStatus[] = {1,2,3,4,5,6,7,8,9};

unsigned long loopTime = 0L;
unsigned long currentTime = 0L;
unsigned long loopTime2 = 0L;
unsigned long currentTime2 = 0L;

int     Angle = 0; 
float   Temperature = 0;
float   Swr_1 = 0;
float   Swr_2 = 0;

const char http_OK[] PROGMEM =
"HTTP/1.0 200 OK\r\n"
"Content-Type: text/html\r\n"
"Pragma: no-cache\r\n\r\n";

const char http_Found[] PROGMEM =
"HTTP/1.0 302 Found\r\n"
"Location: /\r\n\r\n";

const char http_Unauthorized[] PROGMEM =
"HTTP/1.0 401 Unauthorized\r\n"
"Content-Type: text/html\r\n\r\n"
"<h1>401 Unauthorized</h1>";

float GetSWR_1() 
{
  float fwd = (analogRead(A3) * 5.0) / 1023.0;
  float ref = (analogRead(A2) * 5.0) / 1023.0;
	float swr = ( (fwd + ref) / (fwd - ref) );
	return swr;
} 

float GetSWR_2() 
{
  float fwd = (analogRead(A1) * 5.0) / 1023.0;
  float ref = (analogRead(A0) * 5.0) / 1023.0;
	float swr = ( (fwd + ref) / (fwd - ref) );
	return swr;
} 

float GetTEMP()
{
  float tmp =  (analogRead(A4) * 5.0) / 1023.0;
  return(map(tmp, 4.35, 1.94, -10, 50));
}
 
void homePage() {
    bfill.emit_p(PSTR("$F"
    "<meta http-equiv=\"Refresh\" content=\"2\" />"
    "<title>W</title>"
    "<h1><h1>" 
    "<a href=\"?1=$F\">$F</a> <br />"
    "<a href=\"?2=$F\">$F</a> <br />" 
    "<a href=\"?3=$F\">$F</a> <br />"
    "<a href=\"?4=$F\">$F</a> <br />"
    "<a href=\"?5=$F\">$F</a> <br />"
    "<a href=\"?6=$F\">$F</a> <br />"
    "<a href=\"?7=$F\">$F</a> <br />"   
    "<a href=\"?8=$F\">$F</a> <br /><br />"
    "<a href=\"?9=$F\">$F</a> <br /><br />"
    "<font color=\"black\">TEMP </font>$T<br />"
    "<font color=\"black\">SWR1 </font>$T<br />"
    "<font color=\"black\">SWR2 </font>$T<br />"
    "</h1></h1>"),   

    http_OK,

    #ifdef INVERTED_RELAY
      PinStatus[1]?PSTR("on"):PSTR("off"),
      PinStatus[1]?PSTR("<font color=\"green\"><b>ICOM</b></font>"):PSTR("<font color=\"red\"><b>ICOM</b></font>"),
      PinStatus[2]?PSTR("on"):PSTR("off"),
      PinStatus[2]?PSTR("<font color=\"green\"><b>ROTATOR</b></font>"):PSTR("<font color=\"red\"><b>ROTATOR</b></font>"),
      PinStatus[3]?PSTR("on"):PSTR("off"),
      PinStatus[3]?PSTR("<font color=\"green\"><b>OMNI</b></font>"):PSTR("<font color=\"red\"><b>YAGI</b></font>"),
      PinStatus[4]?PSTR("on"):PSTR("off"),
      PinStatus[4]?PSTR("<font color=\"green\"><b>RX AMP</b></font>"):PSTR("<font color=\"red\"><b>RX AMP</b></font>"),
      PinStatus[5]?PSTR("on"):PSTR("off"),
      PinStatus[5]?PSTR("<font color=\"green\"><b>POWER AMP</b></font>"):PSTR("<font color=\"red\"><b>POWER AMP</b></font>"),
      PinStatus[6]?PSTR("on"):PSTR("off"),
      PinStatus[6]?PSTR("<font color=\"green\"><b>ECHO</b></font>"):PSTR("<font color=\"red\"><b>ECHO</b></font>"),   
      PinStatus[7]?PSTR("on"):PSTR("off"),
      PinStatus[7]?PSTR("<font color=\"green\"><b>REPITER</b></font>"):PSTR("<font color=\"red\"><b>REPITER</b></font>"), 
      PinStatus[8]?PSTR("on"):PSTR("off"),
      PinStatus[8]?PSTR("<font color=\"green\"><b>PC POWER</b></font>"):PSTR("<font color=\"red\"><b>PC POWER</b></font>"),
      PinStatus[9]?PSTR("on"):PSTR("off"),
      PinStatus[9]?PSTR("<font color=\"blue\"><b>PC HARD DOWN</b></font>"):PSTR("<font color=\"red\"><b>PC HARD DOWN</b></font>"),
    #else
      PinStatus[1]?PSTR("off"):PSTR("on"),
      PinStatus[1]?PSTR("<font color=\"red\"><b>ICOM</b></font>"):PSTR("<font color=\"green\"><b>ICOM</b></font>"),
      PinStatus[2]?PSTR("off"):PSTR("on"),
      PinStatus[2]?PSTR("<font color=\"red\"><b>ROTATOR</b></font>"):PSTR("<font color=\"green\"><b>ROTATOR</b></font>"),
      PinStatus[3]?PSTR("off"):PSTR("on"),
      PinStatus[3]?PSTR("<font color=\"red\"><b>OMNI</b></font>"):PSTR("<font color=\"green\"><b>YAGI</b></font>"),
      PinStatus[4]?PSTR("off"):PSTR("on"),
      PinStatus[4]?PSTR("<font color=\"red\"><b>RX AMP</b></font>"):PSTR("<font color=\"green\"><b>RX AMP</b></font>"),
      PinStatus[5]?PSTR("off"):PSTR("on"),
      PinStatus[5]?PSTR("<font color=\"red\"><b>POWER AMP</b></font>"):PSTR("<font color=\"green\"><b>POWER AMP</b></font>"),
      PinStatus[6]?PSTR("off"):PSTR("on"),
      PinStatus[6]?PSTR("<font color=\"red\"><b>ECHO</b></font>"):PSTR("<font color=\"green\"><b>ECHO</b></font>"),   
      PinStatus[7]?PSTR("off"):PSTR("on"),
      PinStatus[7]?PSTR("<font color=\"red\"><b>REPITER</b></font>"):PSTR("<font color=\"green\"><b>REPITER</b></font>"), 
      PinStatus[8]?PSTR("off"):PSTR("on"),
      PinStatus[8]?PSTR("<font color=\"red\"><b>PC POWER</b></font>"):PSTR("<font color=\"green\"><b>PC POWER</b></font>"),
      PinStatus[9]?PSTR("off"):PSTR("on"),
      PinStatus[9]?PSTR("<font color=\"red\"><b>PC HARD DOWN</b></font>"):PSTR("<font color=\"blue\"><b>PC HARD DOWN</b></font>"),
    #endif
    Temperature,
    Swr_1,
    Swr_2
    ); 
   
} // End page. 

void setup()
{
  Serial.begin(9600);
    
  //"ethercard" (CS-pin) set to  10.
  if (ether.begin(sizeof Ethernet::buffer, mymac, 10) == 0)
  {
    Serial.println(F("Eth err")); 
  }
  else
  {
    Serial.println(F("Wait DHCP..")); 
  }
 
  if (!ether.dhcpSetup())
  {
    Serial.println(F("DHCP err")); 
  }
  else
  {
    ether.printIp("IP ", ether.myip);
    ether.printIp("Mask ", ether.netmask);
    ether.printIp("GW ", ether.gwip);
    ether.printIp("DNS ", ether.dnsip); 
  }
    
  pinMode(R1,OUTPUT);  digitalWrite(R1, OFF);
  pinMode(R2,OUTPUT);  digitalWrite(R2, OFF);
  pinMode(R3,OUTPUT);  digitalWrite(R3, OFF);
  pinMode(R4,OUTPUT);  digitalWrite(R4, OFF);
  pinMode(R5,OUTPUT);  digitalWrite(R5, OFF);
  pinMode(R6,OUTPUT);  digitalWrite(R6, OFF);
  pinMode(R7,OUTPUT);  digitalWrite(R7, OFF);
  pinMode(R8,OUTPUT);  digitalWrite(R8, OFF);
  pinMode(R9,OUTPUT);  digitalWrite(R9, OFF);
  
  for(int i = 0; i <= 9; i++)
  {
    PinStatus[i] = OFF;
  }
  
  // swr meter
  pinMode(A0, INPUT);
  pinMode(A1, INPUT);
  pinMode(A2, INPUT);
  pinMode(A3, INPUT);
  pinMode(A4, INPUT);
  
  wdt_enable (WDTO_8S); // 8 sec.
  Serial.println(F("Wdt enabled."));
}

// -------------------------------------

void loop()
{ 
  currentTime  = millis();
  currentTime2 = millis();
  
  // ROTATOR Angle
  Angle = map(analogRead(A5), 0, 1024, 0, 450);
  
  // Замер термометра.
  Temperature = GetTEMP();
  
  // SWR
  Swr_1 = GetSWR_1();
  Swr_2 = GetSWR_2();
  
  delay(1); 

  word len = ether.packetReceive();   // check for ethernet packet 
  word pos = ether.packetLoop(len);   // check for tcp packet

  if (pos) {
    bfill = ether.tcpOffset();
    char *data = (char *) Ethernet::buffer + pos;
    if (strncmp("GET /", data, 5) != 0) {
      bfill.emit_p(http_Unauthorized);
    }
    else {
      data += 5;
      
      if (data[0] == ' ') {       
        homePage(); // Return home page.
      }
      else if (strncmp("?1=on ", data, 5) == 0) {
        PinStatus[1] = ON;      
        bfill.emit_p(http_Found);
      }
      else if (strncmp("?2=on ", data, 5) == 0) {
        PinStatus[2] = ON;
        bfill.emit_p(http_Found);
      }
      else if (strncmp("?3=on ", data, 5) == 0) {
        PinStatus[3] = ON;        
        bfill.emit_p(http_Found);
      }
      else if (strncmp("?4=on ", data, 5) == 0) {
        PinStatus[4] = ON;        
        bfill.emit_p(http_Found);
      }
      else if (strncmp("?5=on ", data, 5) == 0) {
        PinStatus[5] = ON;        
        bfill.emit_p(http_Found);
      }
      else if (strncmp("?6=on ", data, 5) == 0) {
        PinStatus[6] = ON;        
        bfill.emit_p(http_Found);
      }
      else if (strncmp("?7=on ", data, 5) == 0) {   
        PinStatus[7] = ON;
        bfill.emit_p(http_Found);
      }
      else if (strncmp("?8=on ", data, 5) == 0) {
        PinStatus[8] = ON;     
        bfill.emit_p(http_Found);
      }
      else if (strncmp("?9=on ", data, 5) == 0) {
        PinStatus[9] = ON;     
        bfill.emit_p(http_Found);
      }

      //-----------------------------------------------------  

     else if (strncmp("?1=off ", data, 6) == 0) {
        PinStatus[1] = OFF;
        bfill.emit_p(http_Found);
      }
      else if (strncmp("?2=off ", data, 6) == 0) {
        PinStatus[2] = OFF;         
        bfill.emit_p(http_Found);
      }
      else if (strncmp("?3=off ", data, 6) == 0) {
        PinStatus[3] = OFF;        
        bfill.emit_p(http_Found);
      }
      else if (strncmp("?4=off ", data, 6) == 0) {
        PinStatus[4] = OFF;        
        bfill.emit_p(http_Found);
      }
      else if (strncmp("?5=off ", data, 6) == 0) {
        PinStatus[5] = OFF;        
        bfill.emit_p(http_Found);
      }
      else if (strncmp("?6=off ", data, 6) == 0) {
        PinStatus[6] = OFF;        
        bfill.emit_p(http_Found);
      }
      else if (strncmp("?7=off ", data, 6) == 0) {
        PinStatus[7] = OFF;        
        bfill.emit_p(http_Found);
      }
      else if (strncmp("?8=off ", data, 6) == 0) {
        PinStatus[8] = OFF;      
        bfill.emit_p(http_Found);
      }
      else if (strncmp("?9=off ", data, 6) == 0) {
        PinStatus[9] = OFF;      
        bfill.emit_p(http_Found);
      }
      else {
        bfill.emit_p(http_Unauthorized);
      }
    }
    ether.httpServerReply(bfill.position());    // send http response.  
  }

  // 0.5 s
  if(currentTime > (loopTime + 500))
  {
    PinStatus[8] = OFF;
    loopTime = currentTime;
  }   

  // 10 s 
  if(currentTime2 > (loopTime2 + 10000))
  {
    PinStatus[9] = OFF;
    loopTime2 = currentTime2;
  }

  digitalWrite(R1, PinStatus[1]);
  digitalWrite(R2, PinStatus[2]);
  digitalWrite(R3, PinStatus[3]);
  digitalWrite(R4, PinStatus[4]);
  digitalWrite(R5, PinStatus[5]);
  digitalWrite(R6, PinStatus[6]);
  digitalWrite(R7, PinStatus[7]);
  digitalWrite(R8, PinStatus[8]);
  digitalWrite(R9, PinStatus[9]);

  if( (PinStatus[8] == ON) || (PinStatus[9] == ON) )
  {
    digitalWrite(R8, ON);
  }
  else
  {
    digitalWrite(R8, OFF);
  }
  
  wdt_reset();
}





